/*
 * TCSS 305Winter 2014
 * Assignment 5b-PowerPaint
 */

package drawingpanelandtools;

import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;

/**
 * This class constructs a tool that is used to make ellipse objects. This class describes the 
 * behavior of shape making when the mouse is pressed, dragged, or released.
 * 
 * @author Brian Dennis
 * @version 20 February 2014
 */
public class EllipseTool extends Tool {

    /** This is the ellipse object that is made when the user releases the mouse click to 
     * finish drawing the shape.
     */
    private PaintShape myReleasedEllipse;
    
    /**
     * This constructor simply calls the constructor of the Tool class.
     * 
     * @param theCanvas the JPanel that the user draws on
     */
    public EllipseTool(final DrawingPanel theCanvas) {
        
        super(theCanvas);
    }
   
    /**
     *This method controls what happens when the mouse click is released. when released, an 
     *ellipse is made from the starting coordinates and ending coordinates of the mouse 
     *pointer.
     * 
     * @param theEvent the event for when the mouse is released
     */
    @Override
    public void ifReleased(final MouseEvent theEvent) {
        
        final Shape ellipse = makeEllipse(myStartDrag.x, myStartDrag.y, theEvent.getX(), 
                                                                             theEvent.getY());
        myReleasedEllipse = new PaintShape(ellipse, myCanvas.getBrushWidth(), 
                                                                    myCanvas.getBrushColor());
    }
    
    /**
     * This method helps construct an ellipse, by using the static methods Math.abs and 
     * Math.min to maintain the integrity of the shape.
     * 
     * @param theX1 starting X coordinate
     * @param theY1 starting Y coordinate
     * @param theX2 ending X coordinate
     * @param theY2 ending Y coordinate
     * @return the newly constructed Ellipse
     */
    private Ellipse2D.Double makeEllipse(final int theX1, final int theY1, final int theX2, 
                                                                            final int theY2) {
        
        return new Ellipse2D.Double(Math.min(theX1, theX2), Math.min(theY1, theY2), 
                                            Math.abs(theX1 - theX2), Math.abs(theY1 - theY2));
    }
    
    /**
     * This method returns the Ellipse segments created from points that have been updated by 
     * the whileDragged method inherited from the Tool class.
     * 
     * @return the current segments of the shape being drawn
     */
    @Override
    public PaintShape returnDraggedShape() {
  
        final Shape ellipse = makeEllipse(myStartDrag.x, myStartDrag.y, myEndDrag.x, 
                                                                                 myEndDrag.y); 
        final PaintShape draggedEllipse = new PaintShape(ellipse, myCanvas.getBrushWidth(), 
                                                                    myCanvas.getBrushColor());
        return draggedEllipse;
    }
    
    /**
     * This method returns the final shape created when the user releases the mouse Button at 
     * the desired location.
     * 
     * @return the final shape
     */
    @Override
    public PaintShape returnReleasedShape() {
        
        return myReleasedEllipse;
    }
}