/*
 * TCSS 305Winter 2014
 * Assignment 5b-PowerPaint
 */

package drawingpanelandtools;

import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.GeneralPath;

/**
 * This class constructs a tool that is used to make pencil objects. This class describes 
 * the behavior of shape making when the mouse is pressed, dragged, or released.
 * 
 * @author Brian Dennis
 * @version 20 February 2014
 */
public class PencilTool extends Tool {

    /** This is the pencil object that is made when the user releases the mouse click to 
     * finish drawing the shape.
     */
    private PaintShape myReleasedPencil;
    
    /** The path used to follow the coordinated of the mouse being dragged. */
    private final GeneralPath myPath;
    
    /**
     * This constructor simply calls the constructor of the Tool class, and initializes the 
     * GeneralPath object.
     * 
     * @param theCanvas the JPanel that the user draws on
     */
    public PencilTool(final DrawingPanel theCanvas) {
        
        super(theCanvas);
        myPath = new GeneralPath();
    }
    
    /**
     * This method moves myPath to where the user clicks.
     * 
     * @param theEvent the event for when the mouse is pressed
     */
    @Override
    public void ifPressed(final MouseEvent theEvent) {
        
        myPath.moveTo(theEvent.getX(), theEvent.getY());
    }
    
    /**
     * This method moves myPath to where the user's mouse pointer is located while dragging.
     * 
     * @param theEvent the event for when the mouse is dragged
     */
    @Override
    public void whileDragged(final MouseEvent theEvent) {
        
        myPath.lineTo(theEvent.getX(), theEvent.getY());
    }
    
    /**
     * This method creates a new shape object out of the path that has been made since the 
     * user has clicked and dragged.
     * 
     * @param theEvent the event for when the mouse click is released
     */
    @Override
    public void ifReleased(final MouseEvent theEvent) {
        
        final Shape pencil = new GeneralPath(myPath);
        myReleasedPencil = new PaintShape(pencil, myCanvas.getBrushWidth(), 
                                                                    myCanvas.getBrushColor());
        myPath.reset();
    }
    
    /**
     * This method returns the pencil segments created from points that have been updated 
     * by the whileDragged method.
     * 
     * @return the current segments of the shape being drawn
     */
    @Override
    public PaintShape returnDraggedShape() {
        
        final Shape pencil = new GeneralPath(myPath);
        final PaintShape draggedPencil = new PaintShape(pencil, myCanvas.getBrushWidth(), 
                                                                    myCanvas.getBrushColor());
        
        return draggedPencil; 
    }
    
    /**
     * This method returns the final shape created when the user releases the mouse Button at 
     * the desired location.
     * 
     * @return the final shape
     */
    @Override
    public PaintShape returnReleasedShape() {
        
        return myReleasedPencil;
    }     
}