/*
 * TCSS 305Winter 2014
 * Assignment 5b-PowerPaint
 */

package drawingpanelandtools;

import java.awt.Point;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;

/**
 * This class is the parent class of all Tools. It's default behavior is that of making a 
 * LineTool. This class is used so objects can be declared as tools in DrawingPanel to reduce 
 * redundancy.
 * 
 * @author Brian Dennis
 * @version 20 February 2014
 */
public class Tool implements ToolInterface {

    /** that starting point of where the user clicks. */
    protected Point myStartDrag; 
    
    /** the ending point of where the user is dragging. */
    protected Point myEndDrag;
    
    /** This is the line object that is made when the user releases the mouse click to 
     * finish drawing the shape.
     */
    protected PaintShape myReleasedLine;
    
    /** the JPanel used to draw images. */
    protected final DrawingPanel myCanvas;
    
 
    /**
     * This constructor simply initializes the myCanvas variable.
     * 
     * @param theCanvas the JFrame that is painted on
     */
    public Tool(final DrawingPanel theCanvas) { 
        
        myCanvas = theCanvas;
    }
    
    /**
     * This method initiates the starting and ending points to be where the user clicks.
     * 
     * @param theEvent the event for when the mouse is pressed
     */
    @Override
    public void ifPressed(final MouseEvent theEvent) {
        
        myStartDrag = new Point(theEvent.getX(), theEvent.getY());
        myEndDrag = myStartDrag;
    }
    
    /**
     * This method updates shape segments as they press the mouse and drag it.
     * 
     * @param theEvent the event for when the mouse is dragged
     */
    public void whileDragged(final MouseEvent theEvent) {
        
        myEndDrag = new Point(theEvent.getX(), theEvent.getY());
    }

    /**
     *This method controls what happens when the mouse click is released. when released, a 
     *line is made from the starting coordinates and ending coordinates of the mouse pointer.
     * 
     * @param theEvent the event for when the mouse is released
     */
    @Override
    public void ifReleased(final MouseEvent theEvent) {
        
        final Shape line = new Line2D.Double(myStartDrag.x, myStartDrag.y, theEvent.getX(), 
                                                                            theEvent.getY());
        myReleasedLine = new PaintShape(line, myCanvas.getBrushWidth(), 
                                                                    myCanvas.getBrushColor());
    }

    
    /**
     * This method returns the line segments created from points that have been updated 
     * by the whileDragged method inherited from the Tool class.
     * 
     * @return the current segments of the shape being drawn
     */
    @Override
    public PaintShape returnDraggedShape() {
        
        final Shape line = new Line2D.Double(myStartDrag.x, myStartDrag.y, myEndDrag.x, 
                                                                                 myEndDrag.y);
        final PaintShape draggedLine = new PaintShape(line, myCanvas.getBrushWidth(), 
                                                                    myCanvas.getBrushColor());
        return draggedLine;
    }
    
    /**
     * This method returns the final shape created when the user releases the mouse Button at 
     * the desired location.
     * 
     * @return the final shape
     */
    @Override
    public PaintShape returnReleasedShape() {
        
        return myReleasedLine;
    }
}