/*
 * TCSS 305Winter 2014
 * Assignment 5b-PowerPaint
 */

package gui;

import drawingpanelandtools.DrawingPanel;
import gui.actions.ColorAction;
import gui.actions.EllipseAction;
import gui.actions.LineAction;
import gui.actions.PencilAction;
import gui.actions.RectangleAction;
import gui.bars.MenuBar;
import gui.bars.ToolBar;
import gui.menus.RedoButton;
import gui.menus.UndoButton;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JRadioButton;
import javax.swing.KeyStroke;

/**
 * This class constructs the JFrame and starts the construction of the various panels.
 * 
 * @author Brian Dennis
 * @version 20 February 2014
 */
//int keyCode = theEvent.getKeyCode();
//String keyText = KeyEvent.getKeyText(keyCode);
//System.out.println(keyText);
//private final Font myFont = new Font("SansSerif", Font.BOLD, 9);
public class PowerPaintGUI {

    /** A ToolKit for getting screen dimensions. */
    private static final Toolkit KIT = Toolkit.getDefaultToolkit();

    /** The Dimension of the screen. */
    private static final Dimension SCREEN_SIZE = KIT.getScreenSize();

    /** The width of the screen. */
    private static final int SCREEN_WIDTH = SCREEN_SIZE.width;

    /** The height of the screen. */
    private static final int SCREEN_HEIGHT = SCREEN_SIZE.height;
    
    /** The frame of the entire GUI. */
    private final JFrame myFrame;
    
    /** A reference to the panel that the user draws on. */
    private final DrawingPanel myCanvas;
    
    /**
     * The constructor of this class which calls the parent class, setting the title of the 
     * JFrame. It also sets the instance fields to a reasonable default.
     */
    public PowerPaintGUI() {
        
        myFrame = new JFrame("TCSS 305 PowerPaint");
        myCanvas = new DrawingPanel();
    }

    /**
     * The main method of this class called by PowerPaintMain to start creating the 
     * JFrame and it's implementation. It does this by creating various Menu and Bar objects
     * and then packing the screen size to fit around the various components.
     */
    public void start() {  
        
        myFrame.setLocation(SCREEN_WIDTH / 4 - myFrame.getWidth() / 4, SCREEN_HEIGHT / 4 
                                                                   - myFrame.getHeight() / 4);
        myFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        myFrame.setLayout(new BorderLayout());
        myFrame.setIconImage(new ImageIcon("icons/paint.gif").getImage());
        myFrame.setVisible(true);
        
        final Action colorAction = new ColorAction(myCanvas);
        final Action pencilAction = new PencilAction(myCanvas);
        final Action lineAction = new LineAction(myCanvas);
        final Action rectangleAction = new RectangleAction(myCanvas);
        final Action ellipseAction = new EllipseAction(myCanvas);
        
        JButton j = new JButton("color");
        j.getInputMap().put(KeyStroke.getKeyStroke("ENTER"),"doEnterAction" );
        j.getActionMap().put( "doEnterAction", colorAction );
        
        
        final MenuBar menuBar = new MenuBar(myFrame,
                                            myCanvas,
                                            /*new JButton(colorAction)*/j, 
                                            new JRadioButton(pencilAction), 
                                            new JRadioButton(lineAction), 
                                            new JRadioButton(rectangleAction), 
                                            new JRadioButton(ellipseAction));
        myFrame.setJMenuBar(menuBar);
        
        
        
        final ToolBar toolBar = new ToolBar(new JButton(colorAction), 
                                            new JRadioButton(pencilAction), 
                                            new JRadioButton(lineAction),
                                            new JRadioButton(rectangleAction),
                                            new JRadioButton(ellipseAction),
                                            new UndoButton(myCanvas),
                                            new RedoButton(myCanvas));
        myCanvas.addPropertyChangeListener(toolBar);
        myFrame.add(toolBar, BorderLayout.SOUTH);
        myFrame.add(myCanvas);
        myFrame.pack();        
    }
}