/*
 * TCSS 305Winter 2014
 * Assignment 5b-PowerPaint
 */

package gui.bars;
import gui.menus.RedoButton;
import gui.menus.UndoButton;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JToolBar;

/**
 * This class constructs the JTolBar found initially on the South section of the GUI.
 * 
 * @author Bad Man Dennis
 * @version 20 February 2014
 */
@SuppressWarnings("serial")
public class ToolBar extends JToolBar implements PropertyChangeListener {
    
    /** This button already has an action connected to it, and represents the color button.*/
    private final JButton myColorButton;
    
    /** This button already has an action connected to it, and represents the pencil button.*/
    private final JRadioButton myPencilButton;
    
    /** This button already has an action connected to it, and represents the line button.*/
    private final JRadioButton myLineButton;
    
    /** This button already has an action connected to it, and represents 
                                                                       the rectangle button.*/
    private final JRadioButton myRectangleButton;
    
    /**This button already has an action connected to it, and represents the ellipse button.*/
    private final JRadioButton myEllipseButton;
    
    /** This group is used to group the various tool buttons. */
    private final ButtonGroup myGroup;
    
    /** The JButton Undo object. */
    private final UndoButton myUndoButton;
    
    /** The JButton Redo object. */
    private final RedoButton myRedoButton;
    
    /**
     * The constructor sets variables to the correct state, and then makes a call to 
     * makeToolButtons().
     * 
     * @param theColorB JButton with an action in it
     * @param thePencilB JRadioButton with an action in it
     * @param theLineB JRadioButton with an action in it
     * @param theRectangleB JRadioButton with an action in it
     * @param theEllipseB JRadioButton with an action in it
     * @param theUndoButton the button for undoing shapes
     * @param theRedoButton thebutton for redoing shapes
     */
    public ToolBar(final JButton theColorB, 
                   final JRadioButton thePencilB, 
                   final JRadioButton theLineB, 
                   final JRadioButton theRectangleB, 
                   final JRadioButton theEllipseB,
                   final UndoButton theUndoButton,
                   final RedoButton theRedoButton) {
        
        super();
        myColorButton = theColorB;
        myPencilButton = thePencilB;
        myLineButton = theLineB;
        myRectangleButton = theRectangleB;
        myEllipseButton = theEllipseB;
        myUndoButton = theUndoButton;
        myRedoButton = theRedoButton;
        
        myGroup = new ButtonGroup();
        
        makeToolButtons();
    }
    
    /**
     * This method adds the JRadioButtons to an associated group and then adds all of the 
     * buttons to this JToolBar.
     */
    private void makeToolButtons() {
        
        myGroup.add(myLineButton);
        myGroup.add(myPencilButton);
        myGroup.add(myRectangleButton);
        myGroup.add(myEllipseButton);
        
        add(myColorButton);
        addSeparator();
        add(myPencilButton);
        add(myLineButton);
        add(myRectangleButton);
        add(myEllipseButton);
        add(myUndoButton);
        add(myRedoButton);
    }
    
    /**
     * This method handles the events for property changes of the Undo and Redo buttons.
     * {@inheritDoc}
     */
    @Override
    public void propertyChange(final PropertyChangeEvent theEvent) {
        
        if ("ToggleUndo".equals(theEvent.getPropertyName())) {
            
            myUndoButton.setEnabled((boolean) theEvent.getNewValue());
        }
        
        if ("ToggleRedo".equals(theEvent.getPropertyName())) {
            
            myRedoButton.setEnabled((boolean) theEvent.getNewValue());
        }
    }
}