/*
 * TCSS 305Winter 2014
 * Assignment 5b-PowerPaint
 */

package gui.menus;
import drawingpanelandtools.DrawingPanel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

/**
 * This class constructs the File JMenu found on the North JMenuBar of the GUI. It also 
 * creates an ActionListener for this JMenu.
 * 
 * @author Bad Man Dennis
 * @version 20 February 2014
 */
@SuppressWarnings("serial")
public class FileMenu extends JMenu {

    /** A reference to the panel that the user draws on. */
    private final DrawingPanel myCanvas;
    
    /** A reference to the frame of the GUI. */
    private final JFrame myFrame;
    
    /** A String used for the "Clear" JMenuItem titles and Action commands. */
    private final String myClearString;
    
    /** A String used for the "Exit" JMenuItem titles and Action commands. */
    private final String myExitString;
    
    /**
     * The constructor initializes variables to a reasonable state and sets a mnemonic for 
     * this JMenu Object. It also makes a method call to makeMenuItems().
     * 
     * @param theMenuName the name of the JMenu
     * @param theCanvas the JPanel you draw on
     * @param theFrame the JFrame of the GUI
     */
    public FileMenu(final String theMenuName, 
                    final DrawingPanel theCanvas, 
                    final JFrame theFrame) {
        
        super();
        this.setText(theMenuName);
        this.setMnemonic(KeyEvent.VK_F);
        myCanvas = theCanvas;
        myFrame = theFrame;
        myClearString = "Clear";
        myExitString = "Exit";
        
        makeMenuItems();
    }

    /**
     * This method creates the 2 JMenuItems found inside the File JMenu, adding Listeners and 
     * mnemonics to them.
     */
    private void makeMenuItems() {
        
        final JMenuItem clearButton = new JMenuItem(myClearString);
        final JMenuItem exitButton = new JMenuItem(myExitString);
        
        clearButton.setMnemonic(KeyEvent.VK_C);
        clearButton.addActionListener(new Listener());
        clearButton.setActionCommand(myClearString);
        
        exitButton.setMnemonic(KeyEvent.VK_X);
        exitButton.addActionListener(new Listener());
        exitButton.setActionCommand(myExitString);
        
        add(clearButton);
        addSeparator();
        add(exitButton);
    }
    
    /**
     * This method Allows The FileMenu to create an Action Listener for it's various 
     * JMenuItems.
     * 
     * @author Brian Dennis
     * @version 20 February 2014
     */
    class Listener implements ActionListener {
        
        /**
         * This method manages what happens when the related JMenuItems are used.
         * 
         * @param theAction the action being analyzed
         */
        public void actionPerformed(final ActionEvent theAction) {
                    
            final String action = theAction.getActionCommand();
            
            if (myClearString.equals(action)) {
                
                myCanvas.clearLists();
                myCanvas.repaint();
                
            } else if (myExitString.equals(action)) {
                
                myFrame.dispatchEvent(new WindowEvent(myFrame, WindowEvent.WINDOW_CLOSING));
            }
        }
    }
}