/*
 * TCSS 305Winter 2014
 * Assignment 5b-PowerPaint
 */

package gui.menus;
import drawingpanelandtools.DrawingPanel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JRadioButton;

/**
 * This class constructs the Options JMenu found on the North JMenuBar of the GUI. It also 
 * creates an ActionListener for this JMenu.
 * 
 * @author Brian Dennis
 * @version 20 February 2014
 */
@SuppressWarnings("serial")
public class OptionsMenu extends JMenu {

    /** A reference to the panel that the user draws on. */
    private final DrawingPanel myCanvas;
    
    /** the group associated with the different levels of pixel thickness. */
    private final ButtonGroup myGroup;
    
    /** An array that holds different levels of pixel thicknesses for drawing shapes. */
    private final int[] myThicknessLevels = {1, 2, 4};
    
    /** A String used for the "Clear" JMenuItem titles and Action commands. */
    private final String myGridString;
    
    /**
     * The constructor initializes variables to a reasonable state and sets a mnemonic for 
     * this JMenu Object. It also makes a method call to makeMenuItems().
     * 
     * @param theMenuName the name of the JMenu
     * @param theCanvas the JPanel you draw on
     */
    public OptionsMenu(final String theMenuName, final DrawingPanel theCanvas) {
        
        super();
        this.setText(theMenuName);
        this.setMnemonic(KeyEvent.VK_O);
        myCanvas = theCanvas;
        myGroup = new ButtonGroup();
        myGridString = "Grid";
        
        makeMenuItems();
    }

    /**
     * This method creates the 2 JMenuItems found inside the Options JMenu, adding Listeners 
     * and mnemonics to them.
     */
    private void makeMenuItems() {
        
        final JCheckBoxMenuItem grid = new JCheckBoxMenuItem(myGridString);
        final JMenu thickness = new JMenu("Thickness");
        
        grid.setMnemonic(KeyEvent.VK_G);
        grid.addActionListener(new Listener());
        grid.setActionCommand(myGridString);
        thickness.setMnemonic(KeyEvent.VK_T);
        
        for (int i = 0; i < myThicknessLevels.length; i++) {
            
            final JRadioButton level = new JRadioButton(String.valueOf(myThicknessLevels[i]));

            level.setMnemonic(KeyEvent.getExtendedKeyCodeForChar((char) ('0' 
                                                                    + myThicknessLevels[i])));
            level.addActionListener(new Listener());
            level.setActionCommand(String.valueOf(myThicknessLevels[i]));
            myGroup.add(level);
            thickness.add(level);
            if (i == 0) {
                level.setSelected(true);
            }
        }
        add(grid);
        addSeparator();
        add(thickness);
    }
    
    /**
     * This method Allows The OptionsMenu to create Action Listeners for it's JMenuItems.
     * 
     * @author Brian Dennis
     * @version 20 February 2014
     */
    class Listener implements ActionListener {
        
        /**
         * This method manages what happens when the related JMenuItems are used.
         * 
         * @param theAction the action being analyzed
         */
        public void actionPerformed(final ActionEvent theAction) {
                    
            final String action = theAction.getActionCommand();
            
            if (myGridString.equals(action)) {
 
                myCanvas.setGridStatus();
                myCanvas.repaint();
            }
            for (final int levels : myThicknessLevels) {
            
                if (String.valueOf(levels).equals(action)) {
                    myCanvas.setBrushWidth(levels);
                }     
            }
        }
    }
}